local mod = MorgDKP2:NewModule("PointsDB")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")

local mdkp = MorgDKP2
mod.modName = "PointsDB"

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("PointsDB", "enUS", true)

if LL then
	LL["Clear player DB"] = true
	LL["Clears all players currently in the database. Does not affect items."] = true
	LL["Database Functions"] = true
	LL["Import database from MorgDKP?"] = true
	LL["Imported MorgDKP DB with %s players and %s items."] = true
	LL["Import default item database?"] = true
	LL["Database transfer complete. %s items were transferred and %s members updated."] = true
	LL["Please delete the ItemData.lua file from the MorgDKP directory now."] = true
	LL["Updating DKP Points.."] = true
	LL["Added alias: %s of %s with %s DKP."] = true
	LL["Added %s players from %s"] = true
	LL["Player database cleared!"] = true
	LL["Ignored items"] = true
	LL["List of currently ignored items.  Click to remove item from ignore list."] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("PointsDB")

local db
local band = bit.band
local bor = bit.bor
local bxor = bit.bxor
local fmt = string.format

local defaults = { 
	profile = {
		
   	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Database Functions"],
		desc = LL["Database Functions"],
		order = 200,
		args = {
			clear = {
				type = "execute",
				name = LL["Clear player DB"],
				desc = LL["Clears all players currently in the database. Does not affect items."],
				func = function() mod:ClearDatabase() end,
				disabled = function() return not mod:IsEnabled() end,
				order = 1
			},
			ignore = {
				type = "select",
				name = LL["Ignored items"],
				desc = LL["List of currently ignored items.  Click to remove item from ignore list."],
				get = false,
				set = function(info, v) 
						table.remove(mdkp.db.profile.ignore, v) 
					end,
				values = function() return mdkp.db.profile.ignore end,
				disabled = function() return not mod:IsEnabled() or not mdkp.db.profile.ignore[1] end,
				order = 5
			},
        	},
}

StaticPopupDialogs["ImportMDKP"] = {
	text = TEXT(LL["Import database from MorgDKP?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:ImportMorgDKPDB()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

StaticPopupDialogs["ImportMorgItems"] = {
	text = TEXT(LL["Import default item database?"]),
	button1 = TEXT(YES),
	button2 = TEXT(NO),
	OnAccept = function()
		mod:ImportMorgItems()
	end,
	timeout = 0,
	whileDead = 1,
	hideOnEscape = 1,
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return nil
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("PointsDB", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["PointsDB"] = true
	if MorgDKP and not db.dbupdated then	--import morgdkp db?
		self:debug("MorgDKP")
		StaticPopup_Show ("ImportMDKP")
		self.importingDB = true
	end
	if MorgDKP2_Items then StaticPopup_Show ("ImportMorgItems") end	--import item db?
	self:ImportDKPInfoFile()
end

function mod:OnDisable()
	db.moduleON["PointsDB"] = nil
end

function mod:ClearDatabase()
	self:debug("Clear DB")
	db.info = { }
	self:out(LL["Player database cleared!"])
end

function mod:ImportMorgDKPDB()
	self:debug("Import MorgDKP")
	local POOL = self:GetPool()
	local update = self:GetServerTime()
	local playercount = 0
	local itemcount = 0
	if not POOL or POOL == L["NONE"] then	--no DKP info imported yet
		db.raid = MorgDKP.db.account.defaults.raid or L["NONE"]
		POOL = db.raid
		db.raidlist = MorgDKP.db.account.defaults.raidlist or L["NONE"]
		db.disenchanter = MorgDKP.db.account.defaults.disenchanter or L["NONE"]
		db.eqDKP = MorgDKP.db.account.defaults.eqDKP
	end
	for index, v in pairs(MorgDKP.db.account.info) do
		local temp = { }
		temp.raidloot = 0
		temp.lastupdate = update
		temp.aliases = v.aliases
		temp.items = {}
		local buffer = {}
		if v.items then
			for _, ID in pairs(v.items) do
				if not buffer[ID] then
					tinsert(temp.items, ID)
					buffer[ID] = true
				end
			end
		end
		if v.class then temp.class = string.upper(v.class) end
		if v[POOL] then 
			temp[POOL] = {points = v[POOL].points, earned = v[POOL].earned, spent = v[POOL].spent}
		else temp[POOL] = {points = 0, earned = 0, spent = 0} end
		db.info[index] = temp
		playercount = playercount + 1
	end
	for id, data in pairs(MorgDKP.db.account.items) do
		if not db.items[id] then
			local temp = {}
			temp.classes = 0
			temp.altclasses = 0
			--if not data.pool then db.items[id].pool = POOL end	shouldnt need default pool
			temp.points = data.points
			for class, val in pairs(data.classes) do
				if val == true then
					bitop = mdkp.Bits[string.upper(class)]
					temp.classes = bor(bitop, temp.classes)
				end
			end
			for class, val in pairs(data.altclasses) do
				if val == true then
					bitop = mdkp.Bits[string.upper(class)]
					temp.altclasses = bor(bitop, temp.altclasses)
				end
			end
			db.items[id] = temp
			itemcount = itemcount + 1
		end
	end
	db.dbupdated = 0
	self:out(fmt(LL["Imported MorgDKP DB with %s players and %s items."], playercount, itemcount))
	mdkp:ClearRaidDB()
	self.importingDB = nil
	self:ImportDKPInfoFile()
end

function mod:ImportMorgItems()
	itemcount = self:ImportItemDB(MorgDKP2_Items)
	self:out(fmt(LL["Database transfer complete. %s items were transferred and %s members updated."], itemcount, 0))
	self:out(LL["Please delete the ItemData.lua file from the MorgDKP directory now."])
	MorgDKP2_Items = nil
end

function mod:ImportItemDB(items)
	local itemcount = 0
	if not db.items then db.items = { } end
	for id, data in pairs(items) do
		if not db.items[id] then
			db.items[id] = data
			itemcount = itemcount + 1
		end
	end
	return itemcount
end

function mod:ImportDKPInfoFile()
	local db = mdkp.db.profile
	if self.importingDB then return end
	local temp = {}
	local count = 1
	if not MorgDKP2_DKP1 then self:DefaultDKP() end
	if not MorgDKP2_DKP1 then return end
	for i = 1,5 do
		local dkp = getglobal("MorgDKP2_DKP" .. i)
		if  dkp then temp[count] =  dkp
			count = count + 1
		end
	end
	local skipupdate = 1
	local timestamp
	if not db.Timestamp or self:TimeStampDiff(temp[1].Timestamp, db.Timestamp) then
		skipupdate = nil
		timestamp = tostring(temp[1].Timestamp)
		if string.len(timestamp) < 12 then timestamp = "0" .. timestamp end
		db.Timestamp = timestamp
		timestamp =  self:GetServerTime()
		self:out(LL["Updating DKP Points.."])
	end
	if not skipupdate then
		db.eqDKP = { }
		db.raidlist = { }
		if not db.raid or db.raid == L["NONE"] then db.raid = temp[1].Pool end
		for i, dkpray in pairs(temp) do
			if dkpray.Pool then
				db.raidlist[i] = dkpray.Pool
				db.eqDKP[dkpray.Pool] = { }
				db.eqDKP[dkpray.Pool]["eqDKPsite"] = dkpray.eqdkpsite
				db.eqDKP[dkpray.Pool]["prefix"] = dkpray.prefix
				if dkpray.Events then
					db.eqDKP[dkpray.Pool]["Events"] = { }
					for event, details in pairs(dkpray.Events) do
						db.eqDKP[dkpray.Pool]["Events"][event] = { }
						db.eqDKP[dkpray.Pool]["Events"][event] = { ["id"] = details.id, ["value"] = details.value}
					end
				end
				if dkpray.Aliases then
					db.eqDKP[dkpray.Pool]["Aliases"] = { }
					for alias, main in pairs(dkpray.Aliases) do
						db.eqDKP[dkpray.Pool]["Aliases"][alias] = main
						self:PlayerinDB(main)
						self:PlayerinDB(alias)
						local present = false
						if not db.info[main].aliases then db.info[main].aliases = { } end
						for numalias = 1 , #db.info[main].aliases do
							if db.info[main].aliases[numalias].alt == alias and db.info[main].aliases[numalias].raid == dkpray.Pool then
								present = true
							end
						end
						if not present then table.insert(db.info[main].aliases , {raid = dkpray.Pool, alt = alias}) end
					end
				end
			end
			if dkpray.Points then
				local count = 0
				for name, v in pairs(dkpray.Points) do
					if not db.info[name] then db.info[name] = { } end
					db.info[name][dkpray.Pool] = {earned = v.earn + v.adj, spent = v.spent, points = v.bal }
					if not db.info[name].class and v.class and v.class ~= "UNKNOWN" then db.info[name].class = v.class end
					db.info[name].raidloot = 0
					db.info[name].lastupdate = timestamp
					count = count + 1
					if db.info[name].aliases then
						for m = 1, #db.info[name].aliases do
							if db.info[name].aliases[m].raid == db.raid then
								local alt = db.info[name].aliases[m].alt
								local mainpoints = db.info[name][dkpray.Pool].points
								if not db.info[alt] then db.info[alt] = { } end
								db.info[alt][dkpray.Pool] = { 	points = mainpoints,
												earned = db.info[name][dkpray.Pool].earned,
												spent = db.info[name][dkpray.Pool].spent }
								db.info[alt].raidloot = 0
								db.info[alt].lastupdate = timestamp
								self:debug(fmt(LL["Added alias: %s of %s with %s DKP."], alt, name, mainpoints)) 
							end
						end
					end
				end
				self:out(fmt(LL["Added %s players from %s"], count, dkpray.Pool))
			end
		end
		MorgDKP2_DKP1 = nil
		MorgDKP2_DKP2 = nil
		MorgDKP2_DKP3 = nil
		MorgDKP2_DKP4 = nil
		MorgDKP2_DKP5 = nil
	end
	self:ToggleFrameUpdate("TooltipLDB", true)
end

function mod:DefaultDKP()
	local db = mdkp.db.profile
	if db.dbupdated == 0 then return end
	local name = UnitName("player")
	MorgDKP2_DKP1 = {
		Timestamp = 010120000800,
		Pool = L["NONE"],
		eqdkpsite = "aqdkp",
		prefix = "eqdkp_",
		Points = {
			[name] = {earn = 1397.00,spent = 405.00,adj = 0.00,bal = 992}
		},
		Aliases = {
			["TestAlias"] = name
		},
		Events = {
			["Void Reaver"] = {id = 77,value = 3.00}
		},
	}
end

function mod:TimeStampDiff(time1, time2)
	local hours1, hours2, min1, min2, day1, day2, month1, month2, year1, year2, total1, total2, timediff

	if time1 and time2 then
		time1 = tostring(time1)
		time2 = tostring(time2)
		if strlen(time1) < 12 then time1 = "0" .. time1 end
		if strlen(time2) < 12 then time2 = "0" .. time2 end
		local _, _,  day1, month1, year1, hours1, min1 = string.find(time1, "(%d%d)(%d%d)(%d%d%d%d)(%d%d)(%d%d)")
		local _, _,  day2, month2, year2, hours2, min2 = string.find(time2, "(%d%d)(%d%d)(%d%d%d%d)(%d%d)(%d%d)")
		total1 = ((((((month1 * 30) + day1) * 24) + hours1) * 60 ) + min1)
		total2 = ((((((month2 * 30) + day2) * 24) + hours2) * 60 ) + min2)
		if year1 > year2 then return true
		elseif total1 > total2 then return true
		else return false
		end
    	end
    	return false
end
