local mod = MorgDKP2:NewModule("Zerosum")
local L = LibStub("AceLocale-3.0"):GetLocale("MorgDKP2")
local media = LibStub("LibSharedMedia-3.0", true)
local BC = LibStub("LibBabble-Class-3.0"):GetLookupTable()

local mdkp = MorgDKP2

--locals
local LL = LibStub("AceLocale-3.0"):NewLocale("Zerosum", "enUS", true)
	
if LL then
	LL["Zerosum DKP"] = true
	LL["Enable"] = true
	LL["Standard zero sum system where member is charged for the item and the charged points are divided between all raid members."] = true
end

local LL = LibStub("AceLocale-3.0"):GetLocale("Zerosum")

mod.modName = LL["Zerosum DKP"]
mod.modref = "Zerosum"

local Player = UnitName("player")
local db
local fmt = string.format

local defaults = { 
	profile = {
		
	},
}

local options = {
		type = "group",
		handler = mod,
		name = LL["Zerosum DKP"],
		desc = LL["Zerosum DKP"],
		order = 360,
		args = {
			enable = {
				type = "toggle",
				name = LL["Enable"],
				desc = LL["Standard zero sum system where member is charged for the item and the charged points are divided between all raid members."],
				get = function() return mdkp.db.profile.modules.Zerosum end,
				set = 	function(info, v) 
						mdkp.db.profile.modules.Zerosum = v
						if v then 
							--mod:Enable()
							mod:DKPChange(mod.modName)
						else 
							mod:Disable()
							mdkp.db.profile.currDKP = nil 	
						end
					end,
				order = 5
			},
		},
}

function mod:GetOptions()
	return options
end

function mod:IsDKP()
	return true
end

function mod:GetName()
	return mod.modName, mod.modref
end

function mod:OnInitialize()
	self.db = mdkp.db:RegisterNamespace("Zerosum", defaults)
	
end

function mod:OnEnable()
	db = mdkp.db.profile
	db.moduleON["Zerosum"] = true
	
end

function mod:OnDisable()
	db.moduleON["Zerosum"] = nil
	--if db.currDKP == "Zerosum" then db.currDKP = nil end						
end

function mod:GetPlayerPoints(name, POOL)
	return db.info[name][POOL].points
end

function mod:GetItemValue(itemid)
	return db.items[itemid].points
end

function mod:ItemRecorded(id, looter, POOL, points, itempoints, link, attendees)
	if looter == L["Bank"] then return end
	points = points - itempoints
	self:AwardItem(id, looter, POOL, points, itempoints, link)
		
	--add zero sum points
	if itempoints == 0 or not attendees then return end
	addpoints = floor((itempoints / #attendees) + 0.5)
	self:AwardDKP(addpoints, 0, POOL, attendees)
end

function mod:ItemRebate(id, looter, POOL, itempoints, link, oldattends, newattends)
	if newattends then		--changing item event
		--remove points from old attendees
		if oldattends then
			local addpoints = floor((itempoints/#oldattends) + 0.5)
			self:AwardDKP(0, addpoints, POOL, oldattends)
		end
		
		--add points to new attendees
		local addpoints = floor((itempoints/#newattends) + 0.5)
		self:AwardDKP(addpoints, 0, POOL, newattends)
	else
		if oldattends and looter ~= L["Bank"] then
			local addpoints = floor((itempoints/#oldattends) + 0.5)
			self:AwardDKP(0, addpoints, POOL, oldattends)
			self:AdjustDKPPoints(looter, looter, POOL, nil, itempoints)
		end
	end
end

function mod:ItemAdjustment(id, looter, POOL, newcost, link, oldcost, attends)
	local diff = oldcost - newcost
	local addpoints = floor((diff/#attends) + 0.5)
	self:AwardDKP(0, addpoints, POOL, attends)
end


function mod:ItemTransfer(id, newmember, oldmember, POOL, newcost, link, oldcost)
	self:AdjustDKPPoints(newmember, oldmember, POOL, newcost, oldcost, link)
end
